@@ -3,13 +3,15 @@
 These estimators will operate in parallel. Their scalability depends
 on the underlying estimators being used.
 """
-from dask_searchcv.model_selection import GridSearchCV, RandomizedSearchCV  # noqa
+from ._search import GridSearchCV, RandomizedSearchCV, compute_n_splits, check_cv
 from ._split import ShuffleSplit, train_test_split
 
 
 __all__ = [
-    'GridSearchCV',
-    'RandomizedSearchCV',
-    'ShuffleSplit',
-    'train_test_split',
+    "GridSearchCV",
+    "RandomizedSearchCV",
+    "ShuffleSplit",
+    "train_test_split",
+    "compute_n_splits",
+    "check_cv",
 ]
